import { getRange, LanguageService } from 'graphql-language-service';
import { toGraphQLPosition, toMonacoRange, toMarkerData, toCompletion, } from './utils';
export class GraphQLWorker {
    constructor(ctx, createData) {
        this._ctx = ctx;
        this._languageService = new LanguageService(createData.languageConfig);
        this._formattingOptions = createData.formattingOptions;
    }
    async doValidation(uri) {
        const document = this._getTextDocument(uri);
        const graphqlDiagnostics = await this._languageService.getDiagnostics(uri, document);
        return graphqlDiagnostics.map(toMarkerData);
    }
    async doComplete(uri, position) {
        const document = this._getTextDocument(uri);
        const graphQLPosition = toGraphQLPosition(position);
        const suggestions = await this._languageService.getCompletion(uri, document, graphQLPosition);
        return suggestions.map(suggestion => toCompletion(suggestion));
    }
    async doHover(uri, position) {
        const document = this._getTextDocument(uri);
        const graphQLPosition = toGraphQLPosition(position);
        const hover = await this._languageService.getHover(uri, document, graphQLPosition);
        return {
            content: hover,
            range: toMonacoRange(getRange({
                column: graphQLPosition.character,
                line: graphQLPosition.line,
            }, document)),
        };
    }
    async doGetVariablesJSONSchema(uri) {
        const document = this._getTextDocument(uri);
        const jsonSchema = await this._languageService.getVariablesJSONSchema(uri, document);
        if (jsonSchema) {
            jsonSchema.$id = 'monaco://variables-schema.json';
            jsonSchema.title = 'GraphQL Variables';
            return JSON.stringify(jsonSchema);
        }
        return null;
    }
    async doFormat(text) {
        const prettierStandalone = await import('prettier/standalone');
        const prettierGraphqlParser = await import('prettier/parser-graphql');
        return prettierStandalone.format(text, {
            ...this._formattingOptions,
            parser: 'graphql',
            plugins: [prettierGraphqlParser],
        });
    }
    async doParse(text) {
        return this._languageService.parse(text);
    }
    _getTextDocument(_uri) {
        const models = this._ctx.getMirrorModels();
        if (models.length > 0) {
            return models[0].getValue();
        }
        return '';
    }
}
export default {
    GraphQLWorker,
};
export function create(ctx, createData) {
    return new GraphQLWorker(ctx, createData);
}
//# sourceMappingURL=GraphQLWorker.js.map