import { LanguageService, } from 'graphql-language-service';
import { Emitter } from 'monaco-editor';
const schemaCache = new Map();
export class MonacoGraphQLAPI {
    constructor({ languageId, schemaConfig, modeConfiguration, formattingOptions, }) {
        this._onDidChange = new Emitter();
        this._onSchemaLoaded = new Emitter();
        this._onSchemaConfigChange = new Emitter();
        this._schemaCache = schemaCache;
        this._schemaConfig = {};
        this._documentString = null;
        this._externalFragmentDefinitions = null;
        this._langService = null;
        this._languageId = languageId;
        if (schemaConfig) {
            this.setSchemaConfig(schemaConfig);
        }
        this.setModeConfiguration(modeConfiguration);
        this.setFormattingOptions(formattingOptions);
        this._schemaCache = new Map();
    }
    get onDidChange() {
        return this._onDidChange.event;
    }
    get onSchemaLoaded() {
        return this._onSchemaLoaded.event;
    }
    get onSchemaConfigChange() {
        return this._onSchemaConfigChange.event;
    }
    get parse() {
        return this._langService.parse;
    }
    get langService() {
        return this._langService;
    }
    get _currentSchemaId() {
        return this.schemaConfig.uri || 'default';
    }
    get _currentSchemaEntry() {
        return this._schemaCache.get(this._currentSchemaId) ?? null;
    }
    getSchemaConfig() {
        return {
            schemaConfig: this.schemaConfig,
            exteralFragmentDefinitions: this.externalFragmentDefinitions ?? undefined,
        };
    }
    async getVariablesJSONSchema(uri, documentText) {
        const JSONSchema = await this._langService?.getVariablesJSONSchema(uri, documentText);
        if (JSONSchema) {
            return JSONSchema;
        }
        return null;
    }
    async loadSchema() {
        try {
            const ls = new LanguageService(this.getSchemaConfig());
            this._langService = ls;
            const schema = await ls.loadSchema();
            if (schema) {
                const schemaEntry = {
                    schema,
                    introspectionJSONString: ls.schemaConfig.introspectionJSONString,
                    documentString: ls.schemaConfig.documentString,
                };
                this.cacheSchema(schemaEntry);
                this._onSchemaLoaded.fire(this);
                return schemaEntry;
            }
        }
        catch (err) {
            console.error('error fetching schema\n', err);
        }
        return null;
    }
    cacheSchema({ introspectionJSONString, documentString, schema, }) {
        this._schemaCache.set(this._currentSchemaId, {
            introspectionJSONString,
            schema,
            documentString,
        });
    }
    get schemaReset() {
        return {
            schema: undefined,
            introspectionJSON: undefined,
            introspectionJSONString: undefined,
            documentAST: undefined,
            documentString: undefined,
        };
    }
    async getSchema() {
        const cachedSchema = this._currentSchemaEntry;
        if (cachedSchema) {
            return cachedSchema;
        }
        return this.loadSchema();
    }
    changeSchema() {
        const cachedSchema = this._schemaCache.get(this.schemaConfig.uri || 'default');
        if (cachedSchema) {
            const ls = new LanguageService(this.getSchemaConfig());
            this._langService = ls;
            this._onSchemaLoaded.fire(this);
        }
        else {
            this.loadSchema().then();
        }
    }
    async reloadSchema() {
        await this.loadSchema();
    }
    get languageId() {
        return this._languageId;
    }
    get modeConfiguration() {
        return this._modeConfiguration;
    }
    get schemaConfig() {
        return this._schemaConfig;
    }
    get formattingOptions() {
        return this._formattingOptions;
    }
    get externalFragmentDefinitions() {
        return this._externalFragmentDefinitions;
    }
    get hasSchema() {
        return Boolean(this._documentString);
    }
    setSchemaConfig(options) {
        this._schemaConfig = options || Object.create(null);
        this._onSchemaConfigChange.fire(this);
    }
    updateSchemaConfig(options) {
        this.setSchemaConfig({
            ...this._schemaConfig,
            ...options,
            requestOpts: {
                ...this._schemaConfig.requestOpts,
                ...options.requestOpts,
                headers: {
                    ...this._schemaConfig.requestOpts?.headers,
                    ...options.requestOpts?.headers,
                },
            },
        });
    }
    setSchemaUri(schemaUri) {
        this.setSchemaConfig({
            ...this._schemaConfig,
            uri: schemaUri,
            ...this.schemaReset,
        });
    }
    setSchema(schema) {
        this._schemaConfig = {
            schema,
            buildSchemaOptions: this._schemaConfig.buildSchemaOptions,
        };
        this._onSchemaLoaded.fire(this);
    }
    setExternalFragmentDefinitions(externalFragmentDefinitions) {
        this._externalFragmentDefinitions = externalFragmentDefinitions;
    }
    setModeConfiguration(modeConfiguration) {
        this._modeConfiguration = modeConfiguration || Object.create(null);
        this._onDidChange.fire(this);
    }
    setFormattingOptions(formattingOptions) {
        this._formattingOptions = formattingOptions || Object.create(null);
        this._onDidChange.fire(this);
    }
}
export const modeConfigurationDefault = {
    documentFormattingEdits: true,
    documentRangeFormattingEdits: false,
    completionItems: true,
    hovers: true,
    documentSymbols: false,
    tokens: false,
    colors: false,
    foldingRanges: false,
    diagnostics: true,
    selectionRanges: false,
};
export const schemaDefault = {
    loadSchemaOnInit: true,
    loadSchemaOnChange: true,
    requestOpts: {
        headers: {
            'Content-Type': 'application/json',
        },
    },
    buildSchemaOptions: {
        assumeValidSDL: true,
    },
};
export const formattingDefaults = {
    prettierConfig: {
        tabWidth: 2,
    },
};
//# sourceMappingURL=api.js.map