import * as monaco from 'monaco-editor';
import { WorkerManager } from './workerManager';
import { language as monarchLanguage, conf, } from 'monaco-editor/esm/vs/basic-languages/graphql/graphql.js';
import * as languageFeatures from './languageFeatures';
export function setupMode(defaults) {
    const disposables = [];
    const providers = [];
    const client = new WorkerManager(defaults);
    const { languageId } = defaults;
    disposables.push(client);
    monaco.languages.setLanguageConfiguration(languageId, conf);
    monaco.languages.setMonarchTokensProvider(languageId, monarchLanguage);
    const worker = (...uris) => {
        try {
            return client.getLanguageServiceWorker(...uris);
        }
        catch (err) {
            throw Error('Error fetching graphql language service worker');
        }
    };
    function registerSchemaLessProviders(newDefaults) {
        const { modeConfiguration } = newDefaults;
        if (modeConfiguration.documentFormattingEdits) {
            providers.push(monaco.languages.registerDocumentFormattingEditProvider(newDefaults.languageId, new languageFeatures.DocumentFormattingAdapter(worker)));
        }
    }
    function registerAllProviders(api) {
        const { modeConfiguration } = api;
        disposeAll(providers);
        if (modeConfiguration.completionItems) {
            providers.push(monaco.languages.registerCompletionItemProvider(api.languageId, new languageFeatures.CompletionAdapter(worker)));
        }
        if (modeConfiguration.diagnostics) {
            providers.push(new languageFeatures.DiagnosticsAdapter(api, worker));
        }
        if (modeConfiguration.hovers) {
            providers.push(monaco.languages.registerHoverProvider(api.languageId, new languageFeatures.HoverAdapter(worker)));
        }
        registerSchemaLessProviders(api);
    }
    defaults.onDidChange(newDefaults => {
        let { modeConfiguration, formattingOptions, schemaConfig } = defaults;
        if (newDefaults.modeConfiguration !== modeConfiguration) {
            modeConfiguration = newDefaults.modeConfiguration;
            registerAllProviders(newDefaults);
        }
        if (newDefaults.formattingOptions !== formattingOptions) {
            formattingOptions = newDefaults.formattingOptions;
            registerSchemaLessProviders(newDefaults);
        }
        if (newDefaults.schemaConfig !== schemaConfig) {
            schemaConfig = newDefaults.schemaConfig;
            registerAllProviders(newDefaults);
        }
    });
    defaults.onSchemaLoaded(api => {
        registerAllProviders(api);
    });
    defaults.onSchemaConfigChange(api => {
        if (api.schemaConfig.loadSchemaOnChange) {
            api.changeSchema();
        }
    });
    disposables.push(asDisposable(providers));
    return asDisposable(disposables);
}
function asDisposable(disposables) {
    return { dispose: () => disposeAll(disposables) };
}
function disposeAll(disposables) {
    while (disposables.length) {
        disposables.pop()?.dispose();
    }
}
//# sourceMappingURL=graphqlMode.js.map